#ifndef DBS_CONSOLE_H_
#define DBS_CONSOLE_H_


#ifdef __cplusplus
extern "C" {
#endif

typedef struct tDbsConsole tDbsConsole;
struct tDbsConsole
{
    u32 linenum;              // Total number of lines of text that can be handled by the console.
    u32 currentline;          // Text was saved up to which line? Continues growing.
    u32 nowcolumn;            // Where on the line is text written?
    u32 displayline;          // Counter for controlling the display position. Which line is being displayed by the upper console?
    u8* linebuf;              // Buffer for storing characrers.
    tDbsConsole* next;        // Link for list used to manage multiple consoles.
    u8 x;                     // X-coordinate on console Units in characters.
    u8 y;                     // Y-coordinate on console Units in characters.
    u8 width;                 // Console width. Units in characters.
    u8 height;                // Console height. Units in characters.
    u8 visible;               // Can it be seen or not? TRUE immediately after create
    u8 linenumber_width;      // How many digits should be used for the line numbers? O immediately after create
    u8 stop_bufferfull;       // Stop after the buffer becomes full? FALSE immediately after create
    u8 default_color;         // Default color
    
};


// Create/delete
void dbs_CreateConsole( tDbsConsole* console, void* linebuf, u32 linebuf_size, int x, int y, int width, int height );
void dbs_DestroyConsole( tDbsConsole* console );
void dbs_CleanupConsole( void );

// Print
void dbs_Puts( const char* fmt, ... );
void dbs_PutsColor( int color, const char* fmt, ... );
void dbs_CPrintf ( tDbsConsole* console, const char* fmt, ... );
void dbs_CVPrintf( tDbsConsole* console, const char* fmt, va_list vlist );
void dbs_PrintColorConsole( tDbsConsole* console, int color, const char* fmt, ... );
void dbs_ClearConsole( tDbsConsole* console );
void dbs_DrawConsole( void );
int  dbs_VSNPrintf( char *dst, size_t len, const char *fmt, va_list vlist );
int  dbs_VNPrintf( char *dst, size_t len, const char *fmt, va_list vlist );

// Scroll
void dbs_ScrollConsole( tDbsConsole* console, int lines );
void dbs_ScrollConsoleToFirstLine( tDbsConsole* console );
void dbs_ScrollConsoleToLastLine( tDbsConsole* console );

void dbs_ScrollConsoleToPrevPage( tDbsConsole* console );
void dbs_ScrollConsoleToNextPage( tDbsConsole* console );

// Set and get functions
void dbs_SetDefaultConsole( tDbsConsole* console );
tDbsConsole* dbs_GetDefaultConsole( void );

void dbs_SetConsoleVisibility( tDbsConsole* console, BOOL show );
BOOL dbs_GetConsoleVisibility( tDbsConsole* console );

void dbs_SetConsoleStopOnBufferFull( tDbsConsole* console,  BOOL stop );
BOOL dbs_GetConsoleStopOnBufferFull( const tDbsConsole* console );

void dbs_SetConsoleLineNumberWidth( tDbsConsole* console, int width );


#ifdef __cplusplus
}/* extern "C"*/
#endif
    

#endif // DBS_CONSOLE_H_
