#include <nitro.h>
#include <ninet/ip.h>
#include <dwc.h>

#include "main.h"
#include "init.h"
#include "utility.h"

#include "dbs.h"

//----------------------------------------------------------------------------
// define
//----------------------------------------------------------------------------
#undef	CONNECT_TYPE_SPECIFY_AP_TYPE	// auto-connection specifying the connection target type
#undef	CONNECT_TYPE_SPECIFY_AP			// auto-connection specifying the connection target SSID

#define GAME_FRAME       1         // Game frames to be anticipated (1 is taken as 1/60)
#define NETCONFIG_USE_HEAP 1
//----------------------------------------------------------------------------
// typedef
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// variable
//----------------------------------------------------------------------------
static KeyControl stKeyCnt;    // Key input control structure

static DWCInetControl stConnCtrl;

static BOOL sPrintOverride; // Overwrite debug output

static u16 card_lock_id;
//----------------------------------------------------------------------------
// prototype
//----------------------------------------------------------------------------
static void NetConfigMain(void);
static BOOL StartIPMain(void);
static void GameWaitVBlankIntr(void);
static void VBlankIntr(void);
//----------------------------------------------------------------------------
// initialized variable
//----------------------------------------------------------------------------

/*---------------------------------------------------------------------------*
  Function for allocating memory
 *---------------------------------------------------------------------------*/
static void*
AllocFunc( DWCAllocType name, u32   size, int align )
{
    void * ptr;
    OSIntrMode old;
    (void)name;
    (void)align;

    old = OS_DisableInterrupts();

    ptr = OS_AllocFromMain( size );

    OS_RestoreInterrupts( old );

#if 0
    OS_TPrintf("##ALLOC:%d:%p\n",size, ptr);

    if ( (u32) ptr == 0x0210f160 )
    {
        OS_TPrintf("#########################leak!\n");
    }
#endif

    return ptr;
}

/*---------------------------------------------------------------------------*
  Function for freeing memory
 *---------------------------------------------------------------------------*/
static void
FreeFunc( DWCAllocType name, void* ptr,  u32 size  )
{
    OSIntrMode old;
    (void)name;
    (void)size;

    if ( !ptr ) return;

    old = OS_DisableInterrupts();

    OS_FreeToMain( ptr );

    OS_RestoreInterrupts( old );
}

/** --------------------------------------------------------------------
  For console output using OS_TPrintf()
  ----------------------------------------------------------------------*/
#ifdef SDK_FINALROM
#undef OS_TVPrintf
#undef OS_TPrintf
#endif
extern void OS_TVPrintf(const char *fmt, va_list vlist );
static char common_buffer[256];        // thread unsafe, but less use of stack

void OS_TVPrintf(const char *fmt, va_list vlist )
{
    (void)OS_VSNPrintf(common_buffer, sizeof(common_buffer), fmt, vlist);
#ifndef SDK_FINALROM
    OS_PutString(common_buffer);
#endif

    if ( sPrintOverride )
    {
        dbs_CVPrintf( NULL, fmt, vlist );
    }
}
#ifdef SDK_FINALROM
extern void OS_TPrintf(const char *fmt, ...);
void OS_TPrintf(const char *fmt, ...)
{
    va_list vlist;

    va_start(vlist, fmt);
    OS_TVPrintf(fmt, vlist);
    va_end(vlist);
}
#endif

/*---------------------------------------------------------------------------*
  Main routine
 *---------------------------------------------------------------------------*/
static u8 s_Work[ DWC_INIT_WORK_SIZE ] ATTRIBUTE_ALIGN( 32 );
void NitroMain ()
{
    int ret;

	OS_Init();
    OS_InitTick();
    OS_InitAlarm();
    RTC_Init();
    CARD_Init();
    ret = OS_GetLockID();
    if (ret == OS_LOCK_ID_ERROR)
    {
        OS_Panic("demo fatal error! OS_GetLockID() failed");
    }
    card_lock_id = (u16)ret;

    // Initial settings for checking for stack overflow
    OS_SetThreadStackWarningOffset(OS_GetCurrentThread(), 0x100);
    
    PXI_Init();
    
    Heap_Init();
	Alarm_Init();

    GX_Init();
    FS_Init( MI_DMA_MAX_NUM ); 

    GX_DispOff();
    GXS_DispOff();

    // Enable V-blank interrupt
    OS_SetIrqFunction(OS_IE_V_BLANK, VBlankIntr);
    (void)OS_EnableIrqMask(OS_IE_V_BLANK);
    (void)OS_EnableIrq();
    (void)OS_EnableInterrupts();
    (void)GX_VBlankIntr(TRUE);
	GX_SetDispSelect(GX_DISP_SELECT_SUB_MAIN);

    // Initialize output
    dbs_DemoInit();
    sPrintOverride = TRUE; // OS_TPrintf() output is linked to the console

    // Starts display
    OS_WaitVBlankIntr();              // Waiting the end of V-Blank interrupt
    GX_DispOn();
    GXS_DispOn();

    // Initialize key input
    InitKeyData(&stKeyCnt);

    //InitGameData();

    // Delete console data for debugging.
    //DWC_Debug_DWCInitError( s_Work, DWC_INIT_RESULT_DESTROY_OTHER_SETTING );

    // Specify debug display level
    DWC_SetReportLevel((unsigned long)(DWC_REPORTFLAG_ALL & ~DWC_REPORTFLAG_QR2_REQ));

    // Initialize DWC library
    ret = DWC_Init( s_Work );
    OS_TPrintf( "DWC_Init() result = %d\n", ret );

    if ( ret == DWC_INIT_RESULT_DESTROY_OTHER_SETTING )
    {
        OS_TPrintf( "Wi-Fi setting might be broken.\n" );
    }

    // Heap usage indicator display ON
    //Heap_SetDebug( TRUE );

    // Set function for allocating memory
    DWC_SetMemFunc( AllocFunc, FreeFunc );
        
	NetConfigMain(); // begin by launching the WiFi connection settings GUI
    // Main loop
    while (1){
	  if(StartIPMain() == TRUE)
			break;
    }
	OS_TPrintf("Turn off the Wi-Fi AP power and DWC library will detect the DCF disconnection.\n");
	
	// Method for maintaining a connection when using the DWC_UpdateConnection function
	while (1){
		if(DWC_UpdateConnection() == FALSE) {
			DWCError dwcerror;
			int errorcode;
			DWCErrorType errortype;
		
			dwcerror = DWC_GetLastErrorEx(&errorcode, &errortype);
			if(dwcerror != DWC_ERROR_NONE) {
				OS_TPrintf("A DCF disconnection is detected by DWC_UpdateConnection()\n");
				OS_TPrintf("DWCError = %d, errorcode = %d, DWCErrorType = %d\n", dwcerror, errorcode, errortype);
				break;
			}
		}
	}
	OS_TPrintf("Turn off the NDS power.\n");
	
	OS_Terminate();
}

/*---------------------------------------------------------------------------*
  Selected GUI
 *---------------------------------------------------------------------------*/
FS_EXTERN_OVERLAY(main_overlay_1);

static void NetConfigMain(void)
{
  //GameMode returnSeq = GAME_MODE_MAIN;

    (void)FS_LoadOverlay( MI_PROCESSOR_ARM9, FS_OVERLAY_ID(main_overlay_1) ) ;

    sPrintOverride = FALSE; // Temporarily restore OS_TPrintf() output to original status.
    dbs_DemoFinalize();

#if defined( USE_AUTHSERVER_RELEASE )
    DWC_SetAuthServer(DWC_CONNECTINET_AUTH_RELEASE);
#endif

#if defined( NETCONFIG_USE_HEAP )
    {
        void* work = OS_Alloc(DWC_UTILITY_WORK_SIZE);

        (void)DWC_StartUtility(work, DWC_LANGUAGE_JAPANESE, DWC_UTILITY_TOP_MENU_FOR_JPN);

        OS_Free(work);
    }
#else
    (void)DWC_StartUtilityEx(DWC_LANGUAGE_JAPANESE, 0);
#endif

    (void)FS_UnloadOverlay( MI_PROCESSOR_ARM9, FS_OVERLAY_ID(main_overlay_1) ) ;

    OS_SetIrqFunction(OS_IE_V_BLANK, VBlankIntr);
    (void)OS_EnableIrqMask(OS_IE_V_BLANK);
    (void)OS_EnableIrq();
    (void)GX_VBlankIntr( TRUE );

    dbs_DemoInit();
    sPrintOverride = TRUE; // OS_TPrintf() output is linked to the console
    OS_WaitVBlankIntr();              // Waiting the end of V-Blank interrupt
    GX_DispOn();
    GXS_DispOn();

    //return returnSeq;
}
/*---------------------------------------------------------------------------*

 *---------------------------------------------------------------------------*/
static BOOL StartIPMain(void)
{
	DWCApInfo apinfo;
	
    DWC_InitInet( &stConnCtrl );

#if defined( USE_AUTHSERVER_RELEASE )
    DWC_SetAuthServer(DWC_CONNECTINET_AUTH_RELEASE);
#endif

	/*-----------------------------------------------------------------------*
	 * for ordinary auto-connections
	 * only DWC_ConnectInetAsync should be called.
	 * For this demo, undef CONNECT_TYPE_SPECIFY_AP and CONNECT_TYPE_SPECIFY_AP_TYPE
	 * To connect by specifying the connection target's AP type,
	 * after calling DWC_SetConnectApType with the AP type passed as an argument,
	 * DWC_ConnectInetAsync should be called.
	 * NB: this will be deleted in the FINALROM version.
	 * For this demo, define CONNECT_TYPE_SPECIFY_AP_TYPE.
	 * To connect by specifying the connection target's AP,
	 * call DWC_DebugConnectInetAsync, passing the connection target AP's SSID and WEP key
	 * as arguments.
	 * For this demo, define CONNECT_TYPE_SPECIFY_AP.
	 *-----------------------------------------------------------------------*/
#ifndef	CONNECT_TYPE_SPECIFY_AP
#ifdef	CONNECT_TYPE_SPECIFY_AP_TYPE
	DWC_SetConnectApType( DWC_APINFO_TYPE_USER0 );	// specify user settings 1
#endif 	// CONNECT_TYPE_SPECIFY_AP_TYPE
	DWC_ConnectInetAsync();
#else
	DWC_DebugConnectInetAsync( "XXXXX", NULL, WCM_WEPMODE_NONE );	// change the argument before using
#endif 	// CONNECT_TYPE_SPECIFY_AP

	// Wait for stable start.
    while ( !DWC_CheckInet() )
    {
        DWC_ProcessInet();

        // Wait for V-blank
        GameWaitVBlankIntr();
    }

    switch ( DWC_GetInetStatus() )
    {
    case DWC_CONNECTINET_STATE_ERROR:
        {
            // error display
            DWCError err;
            int errcode;
            err = DWC_GetLastError( &errcode );

            OS_TPrintf("   Error occurred %d %d.\n", err, errcode );
        }
        DWC_ClearError();

        DWC_CleanupInet();
        break;
    case DWC_CONNECTINET_STATE_FATAL_ERROR:
        {
            // error display
            DWCError err;
            int errcode;
            err = DWC_GetLastError( &errcode );

            OS_Panic("   Error occurred %d %d.\n", err, errcode );
        }
        break;

    case DWC_CONNECTINET_STATE_CONNECTED:
		OS_TPrintf("   CONNECTED!!!.\n");
		if(DWC_GetApInfo(&apinfo) == TRUE) {
			OS_TPrintf("   DWCApInfo.aptype = %d\n", apinfo.aptype);
			OS_TPrintf("   DWCApInfo.area   = %d\n", apinfo.area);
			OS_TPrintf("   DWCApInfo.spotinfo = %s\n", apinfo.spotinfo);
			OS_TPrintf("   DWCApInfo.essid = %s\n", apinfo.essid);
			OS_TPrintf("   DWCApInfo.bssid = %02x:%02x:%02x:%02x:%02x:%02x\n", apinfo.bssid[0], apinfo.bssid[1], apinfo.bssid[2], apinfo.bssid[3], apinfo.bssid[4], apinfo.bssid[5]);
		}
		else {
			OS_TPrintf("   Failed to get DWCApInfo\n");
		}
		return TRUE;
    }
	return FALSE;
}


/*---------------------------------------------------------------------------*
  Function for waiting for V-blank
 *---------------------------------------------------------------------------*/
static void GameWaitVBlankIntr(void)
{
    int i;

    dbs_DemoUpdate();

    // Change the number V-blank waits to match the assumed game frame
    for (i = 0; i < GAME_FRAME; i++){
        OS_WaitIrq(TRUE, OS_IE_V_BLANK);
    }

    Heap_Debug();
    dbs_DemoLoad();

    // Check for stack overflow
    OS_CheckStack(OS_GetCurrentThread());
}


/*---------------------------------------------------------------------------*
  V-blank interrupt function
 *---------------------------------------------------------------------------*/
static void VBlankIntr(void)
{

    OS_SetIrqCheckFlag(OS_IE_V_BLANK);
}
