#ifndef __MAIN_H__
#define __MAIN_H__

// [nakata] Constant used to tell the GameSpy SDK the platform
#ifndef _NITRO
#define _NITRO
#endif

#ifndef GSI_COMMON_DEBUG
#define GSI_COMMON_DEBUG
#endif


#ifdef __cplusplus
extern "C" {
#endif
    

//----------------------------------------------------------------------------
// define
//----------------------------------------------------------------------------
#define GAME_MAX_SAVE_FILE   4   // Number of save files in game (= number of players)
#define GAME_MAX_FRIEND_LIST 32  // Maximum number of friends that can be registered
#define GAME_MAX_PLAYER_NAME 16  // Player name length in game
    

// Game mode enumerator
typedef enum
{
    GAME_MODE_MAIN = 0,    // Offline main game
    GAME_MODE_REG_FRIEND,  // Add/delete friend
    GAME_MODE_LOGIN,       // Login display
    GAME_MODE_CONNECTED,   // After matchmaking ends
    GAME_MODE_NETCONFIG,   // Selected GUI
    GAME_MODE_STARTIP,     // Get IP
    GAME_MODE_STOPIP,      // Release IP
    GAME_MODE_BACKUP,      // Backup-related processing
    GAME_MODE_NUM
} GameMode;


//----------------------------------------------------------------------------
// typedef
//----------------------------------------------------------------------------
// Game sequence function type
// Return value is the next game mode to go to
typedef GameMode (*GameSeqFunc)(void);

// Callback type for making additional displays for each mode
typedef void (*GameAddedDispCallback)(int curIdx, void* param);


typedef struct tagSavedPlayerData
{
    char playerName[GAME_MAX_PLAYER_NAME];  // Player name in game
} SavedPlayerData;

// Friend roster with data added for use by game
typedef struct tagGameFriendList
{
    DWCFriendData keyList[GAME_MAX_FRIEND_LIST];  // DWC format friend roster
    char playerName[GAME_MAX_FRIEND_LIST][GAME_MAX_PLAYER_NAME];  // Player name
} GameFriendList;

// Game save data structure
typedef struct tagGameSaveData
{
    SavedPlayerData playerData[GAME_MAX_SAVE_FILE];
    int  profileID;        // User-specific profile ID
    BOOL isValidProfile;   // Is the profile ID valid?
    GameFriendList friendList;  // Friend roster with data added for use by game
} GameSaveData;


// Game sequence control list element structure
typedef struct tagGameMenuElement
{
    const char* msg;  // Corresponding message
    GameMode mode;    // Corresponding game mode. Specify GAME_MODE_NUM when the mode doesn't change
    BOOL blocking;    // Is it necessary to disable operations until status has changed when making selections?
} GameMenuElement;

// Game sequence control list structure
typedef struct tagGameSequence
{
    const char* modeName;  // Mode name
    GameSeqFunc seqFunc;   // Corresponding function
    GameAddedDispCallback dispCallback;  // Corresponding callback for making additional display
    int numMenu;           // Number of menus
    GameMenuElement menuList[8];  // Menu list
} GameSequence;


typedef struct tagGameUserData
{
    int  playerIdx;       // Player index in the game
    char playerName[GAME_MAX_PLAYER_NAME];  // Player names in game currently being played
    int  profileID;       // User-specific profile ID
    BOOL isValidProfile;  // Is the profile ID valid?
} GameUserData;

// Game control data structure
typedef struct tagGameControl
{
    GameUserData userData;  // User data during play
    GameMode mode;          // Current game mode
    BOOL blocking;          // Operations disabled mode if TRUE (wait for change in network status)
    const char* essID;      // ESSID of the AP to be used
    GameFriendList friendList;  // Friend roster with data added for use by game
    u8  friendStatus[GAME_MAX_FRIEND_LIST];  // Friend communication status. The index corresponds to an entry on the friend roster
} GameControl;

// Additional key data structure for peer matchmaking without a friend specification
typedef struct tagGameMatchExtKeys
{
    u8  keyID;        // Key ID used for matchmaking
    u8  isStr;        // 0: Value is an in, 1: Value is a text string
    u16 pad;          // Padding
    char keyStr[16];  // Key string used for matchmaking
    char svalue[16];  // Value corresponding to key (string)
    int ivalue;       // Value corresponding to key (int type)
} GameMatchExtKeys;


//----------------------------------------------------------------------------
// function
//----------------------------------------------------------------------------


#ifdef __cplusplus
}
#endif

#endif  // __MAIN_H__
