#include <nitro.h>
#include <ninet/ip.h>
#include <dwc.h>

#include "main.h"
#include "init.h"
#include "utility.h"
#include "userdata.h"
#include "screen.h"
#include "mp.h"

#include "dbs.h"

/** --------------------------------------------------------------------
  defines
  ----------------------------------------------------------------------*/
#define GAME_NAME        "dwctest" // Game name used in this sample
#define GAME_SECRET_KEY  "d4q9GZ"  // Secret key used in this sample
#define GAME_PRODUCTID   10824      // Product ID used in this sample

/** --------------------------------------------------------------------
  globals
  ----------------------------------------------------------------------*/
KeyControl		g_KeyCtrl;

/** --------------------------------------------------------------------
  statics
  ----------------------------------------------------------------------*/
static u8 s_Work[ DWC_INIT_WORK_SIZE ] ATTRIBUTE_ALIGN( 32 );
static DWCFriendsMatchControl	s_FMCtrl;
static DWCInetControl			s_ConnCtrl;
static DWCFriendData stSampleFriendData ATTRIBUTE_ALIGN( 32 );// friend data for sample MP exchange
static u16* s_SendBuf ATTRIBUTE_ALIGN(32);                  // Buffer for sending data
static char stFriendKey[sizeof(DWCFriendData)];             // Friend key
static int  s_state  = state_init;
static int  s_msgSeq;

static const char* s_messages[msg_seq_last][6] =
{
    {
        NULL, NULL, NULL, NULL, NULL, NULL
    },
    {
        "The Wi-Fi ID may have been\nerased.\nPlease check your access-point\nsettings with Nintendo WFC\nsetup.",
        "continue",
        NULL, NULL, NULL, NULL
    },
    {
        "Connect to Nintendo WFC?",
        "yes",
        "no", NULL, NULL, NULL
    },
    {
        "Nintendo WFC treats your\nGame Card and Nintendo DS as\na set.\nSave this Nintendo DS system's\nNintendo WFC configuration to\nthis Game Card and connect to\nNintendo WFC?",
        "yes",
        "no", NULL, NULL, NULL
    },
    {
        "The Wi-Fi ID from this \nNintendo DS has been saved to \nthe Game Card.\nFrom now on, please use this\nNintendo DS when attempting to\nconnect to Nintendo WFC.",
        "continue",
        NULL, NULL, NULL, NULL
    },
    {
        "Unable to connect to Nintendo \nWFC because the Wi-Fi ID from \nthe Nintendo DS and the Game \nCard do not match.",
        "continue",
        NULL, NULL, NULL, NULL
    },
    {
        "When you update your Wi-Fi ID,\nyou will lose any data used \nto connect to Nintendo WFC up \nto this point.\nUpdate the Wi-Fi ID saved to \nthis Game Card anyway?",
        "yes",
        "no", NULL, NULL, NULL
    },
    {
        "Do you still want to update?",
        "yes",
        "no", NULL, NULL, NULL
    },
    {
        "Connecting to Nintendo WFC...\nPlease wait a moment...",
        "success",
        "error", NULL, NULL, NULL
    },
};

/** --------------------------------------------------------------------
  static functions
  ----------------------------------------------------------------------*/
static int  update_init( void );
static int  update_initMPData( void );
static int  update_changeMPData( void );
static int  update_initFriendKey( void );
static int  update_changeFriendKey( void );
static int  update_readyToConnect( void );
static BOOL InputNumber( void );
static void DispMenuMsg( const u8 figure );
static int  do_msgSeqInit( void );
static int  do_msgSeq0003( void );
static int  do_msgSeq0004( void );
static int  do_msgSeq0005( void );
static int  do_msgSeq0006( void );
static int  do_msgSeq0007( void );
static int  do_msgSeq0008( void );
static int  do_msgSeq0009( void );

/*---------------------------------------------------------------------------*
  V-blank interrupt function
 *---------------------------------------------------------------------------*/
static void VBlankIntr(void)
{
    //---- upload pseudo screen to VRAM
    DC_FlushRange(gScreen, sizeof(gScreen));
    GX_LoadBG0Scr(gScreen, 0, sizeof(gScreen));

    OS_SetIrqCheckFlag(OS_IE_V_BLANK);
}


/** --------------------------------------------------------------------
  Initialization processing
  ----------------------------------------------------------------------*/
static void
initFunctions( void )
{
	OS_Init();
    OS_InitTick();
    OS_InitAlarm();
    RTC_Init();
    CARD_Init();
    OS_SetThreadStackWarningOffset(OS_GetCurrentThread(), 0x100);
    PXI_Init();
    GX_Init();
    FS_Init( MI_DMA_MAX_NUM ); 

    GX_DispOff();
    GXS_DispOff();

    // Enable V-blank interrupt
    OS_SetIrqFunction(OS_IE_V_BLANK, VBlankIntr);
    (void)OS_EnableIrqMask(OS_IE_V_BLANK);
    (void)OS_EnableIrq();
    (void)OS_EnableInterrupts();
    (void)GX_VBlankIntr(TRUE);
	GX_SetDispSelect(GX_DISP_SELECT_SUB_MAIN);

    Heap_Init();
    Heap_SetDebug( TRUE );

    // Initialize output
    InitScreen();
    dbs_DemoInit();
    DTDB_SetPrintOverride( TRUE );

    // Initialize key input
    InitKeyData( &g_KeyCtrl );

    // Starts display
    OS_WaitVBlankIntr();              // Waiting the end of V-Blank interrupt
    GX_DispOn();
    GXS_DispOn();
}

/** --------------------------------------------------------------------
  function called once per frame
  ----------------------------------------------------------------------*/
static void
update( void )
{
    switch ( s_state )
    {
    case state_init:			s_state = update_init();			  break;
    case state_initMPData:      s_state = update_initMPData();        break;
    case state_changeMPData:    s_state = update_changeMPData();      break;
    case state_initFriendKey:   s_state = update_initFriendKey();     break;
    case state_changeFriendKey: s_state = update_changeFriendKey();   break;
    case state_readyToConnect:	s_state = update_readyToConnect();	  break;
    }
}

/** --------------------------------------------------------------------
  Initial state
  ----------------------------------------------------------------------*/
static int
update_init( void )
{
    DispMessage( "INIT MODE", "save", "create userdata", "change MPdata", "change friendkey", "friend state" );

    if ( g_KeyCtrl.trg & PAD_BUTTON_A )
    {
        // Begins preparations to make a Wi-Fi connection using the A button.
        s_msgSeq = msg_seq_init;
        return state_readyToConnect;
    }
    else if ( g_KeyCtrl.trg & PAD_BUTTON_B )
    {
        // Deletes backup data using the B button
        DTUD_DeleteUserData();
        DTUD_DeleteFriendList();
        DTUD_Init();
        
        OS_TPrintf( "Delete and Create UserData.\n" );
    }
    else if( g_KeyCtrl.trg & PAD_BUTTON_X )
    {
        // Switches to friend data registration mode in MP with the X button
        DispMessage( "CHANGE MP FRIENDDATA?", "yes", "no", NULL, NULL, NULL );
        return state_initMPData;
    }
    else if( g_KeyCtrl.trg & PAD_BUTTON_Y )
    {
        // friend data registration mode in friend registration key with the Y button
        DispMessage( "CHANGE FRIEND KEY?", "yes", "no", NULL, NULL, NULL );
        return state_initFriendKey;
    }
    else if( g_KeyCtrl.trg & PAD_BUTTON_L )
    {
        DTUD_DispFriendList();
    }

    return state_init;
}

/*---------------------------------------------------------------------------*
  Name:         update_initMPData

  Description:  Creates and sets data for MP communications

  Arguments:    NONE.

  Returns:      Returns state_changeMPData when making an exchange
                Returns state_init when cancelling an exchange
 *---------------------------------------------------------------------------*/
static int
update_initMPData( void )
{
    DWCUserData*   s_UserData;
    DWCFriendData* s_FriendList;
    
    if ( g_KeyCtrl.trg & PAD_BUTTON_A )
    {
        // Create data for MP communications
        s_UserData = DTUD_GetUserData();
        DWC_CreateExchangeToken(s_UserData, &stSampleFriendData);
        s_SendBuf = (u16*)(&stSampleFriendData);
        
        // Sets MP data
        s_FriendList = DTUD_GetFriendList();
        MP_SetData(s_SendBuf, s_FriendList);
        return state_changeMPData;
    }
    else if( g_KeyCtrl.trg & PAD_BUTTON_B )
    {
        return state_init;
    }
    
    return state_initMPData;
}

/*---------------------------------------------------------------------------*
  Name:         update_changeMPData

  Description:  Exchanges friend data in MP and registers the exchanged data.

  Arguments:    NONE.

  Returns:      Returns state_init when the exchange is complete or cancelled.
                Returns state_changeMPData if it has not finished.
 *---------------------------------------------------------------------------*/
static int
update_changeMPData( void )
{
    DWCFriendData* s_FriendList;
    int  friendIdx;
    int  i,j;

    // Starts MP communications.
    if( MP_Match() )
    {
        // Ends MP communications
        s_FriendList = DTUD_GetFriendList();
        friendIdx = DTUD_GetAvailableFriendListIndex();// Omit identical data
        for(i = 0; i < friendIdx; i++){
            for(j = i+1; j < friendIdx; j++){
                if(DWC_IsEqualFriendData( (const DWCAccFriendData*)&s_FriendList[i], (const DWCAccFriendData*)&s_FriendList[j])){
                    MI_CpuClear8(&s_FriendList[j], sizeof(DWCFriendData));
                }
            }
        }
        MP_Finish();
        DTUD_DispFriendList();
        return state_init;
    }
    return state_changeMPData;
}

/*---------------------------------------------------------------------------*
  Name:         update_initFriendKey

  Description:  performs friends registration key initialization

  Arguments:    NONE.

  Returns:      Returns state_init if the friend list is full and registration is not possible or registration is cancelled.
                Returns state_change if exchange registration starts.
 *---------------------------------------------------------------------------*/
static int
update_initFriendKey( void )
{
    int  friendIdx = -1;
    
    if ( g_KeyCtrl.trg & PAD_BUTTON_A )
    {
        // Clears the friend registration key to '0'
        MI_CpuFill8( stFriendKey, '0', sizeof(stFriendKey));
        
        // Check empty space on the friend list
        friendIdx = DTUD_GetAvailableFriendListIndex();
        if ( friendIdx == -1 )
        {
            OS_TPrintf( "Failed to register new friend. Friend list is full.\n" );
            return state_init;
        }
        else
        {
            return state_changeFriendKey;
        }
    }
    else if( g_KeyCtrl.trg & PAD_BUTTON_B )
    {
        return state_init;        
    }
    
    return state_initFriendKey;

}

/*---------------------------------------------------------------------------*
  Name:         update_changeFriendKey

  Description:  Runs friend registration key input and friend registration.

  Arguments:    NONE.

  Returns:      Returns state_init if registration is complete.
                Returns state_changeFriendKey if it has not finished.
 *---------------------------------------------------------------------------*/
static int
update_changeFriendKey( void )
{
    DWCUserData*   s_UserData;
    DWCFriendData* s_FriendList;
    int  friendIdx = -1;
    u8   maxFigure = 12;// Maximum 12 digits for friend registration key

    if ( g_KeyCtrl.trg & PAD_BUTTON_B )
    {
        // Pressing the B button cancels input
        DispMessage( "STATE : ONLINE", "save data", "shutdown", "change friend key", "delete friend data", "friend state" );
        return state_init;
    }
    // friend registration key input
    else if( InputNumber() )
    {
        // Validity check of friend registration key
        s_UserData = DTUD_GetUserData();
        if( DWC_CheckFriendKey( s_UserData, DWC_Acc_NumericKeyToFriendKey( stFriendKey )))
        {
            // Register friend using friend registration key
            s_FriendList = DTUD_GetFriendList();
            friendIdx = DTUD_GetAvailableFriendListIndex();
            DWC_CreateFriendKeyToken( &s_FriendList[friendIdx], DWC_Acc_NumericKeyToFriendKey( stFriendKey ) );
        }
        else
        {
            OS_TPrintf( "Invalid Friend Code!\n" );
        }
        DTUD_DispFriendList();
        return state_init;
    }
    
    return state_changeFriendKey;
}

/** --------------------------------------------------------------------
  Prepares to connect a Wi-Fi connection
  ----------------------------------------------------------------------*/
static int
update_readyToConnect( void )
{
    // Displays messages for each sequence
    DispMessage( s_messages[s_msgSeq][0], s_messages[s_msgSeq][1], s_messages[s_msgSeq][2], s_messages[s_msgSeq][3], s_messages[s_msgSeq][4], s_messages[s_msgSeq][5]);

    switch ( s_msgSeq )
    {
    case msg_seq_init:			s_msgSeq = do_msgSeqInit();			break;
    case msg_seq_0003:			s_msgSeq = do_msgSeq0003();			break;
    case msg_seq_0004:			s_msgSeq = do_msgSeq0004();			break;
    case msg_seq_0005:			s_msgSeq = do_msgSeq0005();			break;
    case msg_seq_0006:			s_msgSeq = do_msgSeq0006();			break;
    case msg_seq_0007:			s_msgSeq = do_msgSeq0007();			break;
    case msg_seq_0008:			s_msgSeq = do_msgSeq0008();			break;
    case msg_seq_0009:			s_msgSeq = do_msgSeq0009();			break;
    }

    if ( s_msgSeq == msg_seq_exit )
    {
        return state_init;
    }
    else
    {
        return state_readyToConnect;
    }
}

/** --------------------------------------------------------------------
  Sequence operation corresponding to the flow at the time of connection
  ----------------------------------------------------------------------*/
static int
do_msgSeqInit( void )
{
    if ( DWC_CheckHasProfile( DTUD_GetUserData() ) )
    {
        // When there is already a GameSpy profile.
        if ( DWC_CheckValidConsole( DTUD_GetUserData() ) )
        {
            // When the user data on the DS device coincides with that on the DS card
            return msg_seq_0003;
        }
        else
        {
            // When the user data on the DS device does not coincide with that on the DS card
            OS_TPrintf( "UserData is not Valid Console.\n" );

            return msg_seq_0006;
        }
    }
    else
    {
        // When there has not yet been even one login (or when data could not be saved after login)
        // it proceeds as-is to the connection sequence.
        // In this sample, no actual Wi-Fi connection is made,
        // so without being able to receive a certified login ID, it will come here every time.
        return msg_seq_0004;
    }

    return msg_seq_init;
}

static int
do_msgSeq0003( void )
{

    if ( g_KeyCtrl.trg & PAD_BUTTON_A )
    {
        // Starts connecting to the Wi-Fi connection with the A button
        return msg_seq_0009;
    }
    else if ( g_KeyCtrl.trg & PAD_BUTTON_B )
    {
        // Interrupts the operation and returns to the beginning with the B button
        return msg_seq_exit;
    }

    return msg_seq_0003;
}

static int
do_msgSeq0004( void )
{
    
    if ( g_KeyCtrl.trg & PAD_BUTTON_A )
    {
        // Starts connecting to the Wi-Fi connection with the A button
        return msg_seq_0009;
    }
    else if ( g_KeyCtrl.trg & PAD_BUTTON_B )
    {
        // Interrupts the operation and returns to the beginning with the B button
        return msg_seq_exit;
    }

    return msg_seq_0004;
}

static int
do_msgSeq0005( void )
{

    if ( g_KeyCtrl.trg & PAD_BUTTON_A )
    {
        // Considers the connection complete and returns to the beginning with the A button
        OS_TPrintf( "Connected!\n" );
        return msg_seq_exit;
    }

    return msg_seq_0005;
}

static int
do_msgSeq0006( void )
{

    if ( g_KeyCtrl.trg & PAD_BUTTON_A )
    {
        // Proceeds to the next one with the A button
        return msg_seq_0007;
    }

    return msg_seq_0006;
}

static int
do_msgSeq0007( void )
{

    if ( g_KeyCtrl.trg & PAD_BUTTON_A )
    {
        // Proceeds to the next one with the A button
        return msg_seq_0008;
    }
    else if ( g_KeyCtrl.trg & PAD_BUTTON_B )
    {
        // Interrupts the operation and returns to the beginning with the B button
        return msg_seq_exit;
    }

    return msg_seq_0007;
}

static int
do_msgSeq0008( void )
{

    if ( g_KeyCtrl.trg & PAD_BUTTON_A )
    {
        // Starts connecting to the Wi-Fi connection with the A button
        return msg_seq_0009;
    }
    else if ( g_KeyCtrl.trg & PAD_BUTTON_B )
    {
        // Interrupts the operation and returns to the first question with the B button
        return msg_seq_0006;
    }

    return msg_seq_0008;
}

static int
do_msgSeq0009( void )
{
    
    if ( g_KeyCtrl.trg & PAD_BUTTON_A )
    {
        // Considers the operation to be successful with no errors by pressing the A button
        DTUD_SaveFriendList();
        OS_TPrintf( "Saved FriendData.\n");
        if ( DTUD_CheckUpdateUserData() )
        {
            // Saves if user data has been updated
            DTUD_SaveUserData();

            OS_TPrintf( "Saved UserData.\n");
            return msg_seq_0005;
        }
        else
        {
            // Considers the connection to be complete and returns to the beginning
            OS_TPrintf( "Connected!\n" );
            return msg_seq_exit;
        }
    }
    else if ( g_KeyCtrl.trg & PAD_BUTTON_B )
    {
        // Considers there to be an error and returns to the beginning with the B button
        OS_TPrintf( "Error occured!\n" );
        return msg_seq_exit;
    }

    return msg_seq_0009;
}

/*---------------------------------------------------------------------------*
  Name:         InputNumber

  Description:  performs friends registration key numerical value input

  Arguments:    NONE.

  Returns:      Returns the friend registration key input.
                key_change: input complete
                key_set   : during input
 *---------------------------------------------------------------------------*/
static BOOL InputNumber( void )
{
    static u8  figure = 0;
    u8 maxFigure = 12;

    DispMenuMsg(figure);
    
    if (g_KeyCtrl.trg & PAD_BUTTON_A)
    {
        figure = 0;
        return TRUE;
    }
    else if (g_KeyCtrl.trg & PAD_KEY_UP)
    {
        // Increment numeric value during selection using the up/down buttons on the crosskey
        stFriendKey[figure]++;
        if(stFriendKey[figure] > '9')
        {
            stFriendKey[figure] = '0';
        }
        // Displays numerical values during input
        DispMenuMsg(figure);
    }
    else if (g_KeyCtrl.trg & PAD_KEY_DOWN)
    {
        // Increment numeric value during selection using the up/down buttons on the crosskey
        stFriendKey[figure]--;
        if(stFriendKey[figure] < '0')
        {
            stFriendKey[figure] = '9';
        }
        // Displays numerical values during input
        DispMenuMsg(figure);
    }
    else if (g_KeyCtrl.trg & PAD_KEY_LEFT)
    {
        // Move the number of selected digits using the left/right buttons on the crosskey
        if (figure > 0 && figure <= maxFigure-1 ) figure--;
        // Displays numerical values during input
        DispMenuMsg(figure);
    }
    else if (g_KeyCtrl.trg & PAD_KEY_RIGHT)
    {
        // Move the number of selected digits using the left/right buttons on the crosskey
        if (figure >= 0 && figure < maxFigure-1 ) figure++;
        // Displays numerical values during input
        DispMenuMsg(figure);
    }
    return FALSE;
}

/*---------------------------------------------------------------------------*
  Name:         DispMenuMsg

  Description:  Displays the numerical value during friend registration

  Arguments:    figure    - number of digits during input
                maxFigure - maximum number of digits

  Returns:      NONE.
 *---------------------------------------------------------------------------*/
static void DispMenuMsg( const u8 figure )
{
    int i;
    u8 maxFigure = 12;

    int x = 16;

    DispMessage( "NUMBER INPUT", "set data", "cancel", NULL, NULL, NULL );
    
    // Currently entering a numeric value
    // Register friend friend registration key being input
    PrintString( 1, 15, 0xf, "Set friend key:");
    for(i = 0; i < maxFigure; i++){
        PrintString( (s16)(x+i), 15, 0xf, "%c", stFriendKey[i] );
        if(i%4 == 3 && i != maxFigure - 1)
        {
            x++;
            PrintString( (s16)(x+i), 15, 0xf, "-" );
        }
    }
    // Display an underline to indicate the numeric value currently being selected
    PrintString( (s16)(16+figure+figure/4), 16, 0xf, "-" );
}

/*---------------------------------------------------------------------------*
  Name:         DispMessage

  Description:  Displays general messages

  Arguments:    main_msg - current mode
                msg_a    - operation when the A button is pressed
                msg_b    - operation when the B button is pressed
                msg_x    - operation when the X button is pressed
                msg_y    - operation when the Y button is pressed
                msg_l    - operation when the L button is pressed

  Returns:      NONE.
 *---------------------------------------------------------------------------*/
void
DispMessage( const char* main_msg, const char* msg_a, const char* msg_b, const char* msg_x, const char* msg_y,  const char* msg_l )
{
    s16 x = 1;
    s16 y = 7;

    ClearScreen();
    
    PrintString( 0, 0, 0xf, "================================" );
    PrintString( 2, 1, 0xf, "*** USER DATA SAMPLE ***" );
    PrintString( 0, 2, 0xf, "================================" );
    
    if ( main_msg == NULL ) return;

    PrintString( 0, y, 0xf, "--------------------------------" );
    y++;
    // Displays a message
    while ( *main_msg != '\0' )
    {
        if ( *main_msg == '\n' )
        {
            x = 1;  y++;
        }
        else
        {
            PrintString( x, y, 0xf, "%c", *main_msg );
            x++;
        }
        main_msg++;
    }
    y++;
    PrintString( 0, y, 0xf, "--------------------------------" );

    y += 2;

    // Displays an explanation of the A button and explanations of the B, X, and Y buttons
    if ( msg_a != NULL )
    {
        PrintString( 1, y, 0xf, "A BUTTON:%s", msg_a );
    }
    y++;
    if ( msg_b != NULL )
    {
        PrintString( 1, y, 0xf, "B BUTTON:%s", msg_b );
    }
    y += 2;
    if ( msg_x != NULL )
    {
        PrintString( 1, y, 0xf, "X BUTTON:%s", msg_x );
    }
    y++;
    if ( msg_y != NULL )
    {
        PrintString( 1, y, 0xf, "Y BUTTON:%s", msg_y );
    }
    y += 2;
    if ( msg_l != NULL )
    {
        PrintString( 1, y, 0xf, "L BUTTON:%s", msg_l );
    }
}

/** --------------------------------------------------------------------
  ----------------------------------------------------------------------*/
/** --------------------------------------------------------------------
  Status display function called every frame.
  ----------------------------------------------------------------------*/
void
DrawConsole( void )
{
    static int i = 0;
    char loop[] = "/|\\-";

    dbs_Print( 30,0, "%c", loop[((i)>>2)&3] );

    i++;
}

/** --------------------------------------------------------------------
  Entrypoint
  ----------------------------------------------------------------------*/
void NitroMain ()
{
    initFunctions();

    // Specify debug display level
    DWC_SetReportLevel((u32)( DWC_REPORTFLAG_ALL &
                              ~DWC_REPORTFLAG_QR2_REQ ));
    
    // Initialize DWC library
    if ( DWC_Init( s_Work ) == DWC_INIT_RESULT_DESTROY_OTHER_SETTING )
    {
        // The NVRAM of the DS device is destroyed
        DispMessage( s_messages[s_msgSeq][0], s_messages[s_msgSeq][1], s_messages[s_msgSeq][2], s_messages[s_msgSeq][3], s_messages[s_msgSeq][4], s_messages[s_msgSeq][5]);
        
        while (1)
        {
            DrawConsole();
            ReadKeyData();

            // Proceeds to the next one with the A button
            if ( g_KeyCtrl.trg & PAD_BUTTON_A ) break;

            dbs_DemoUpdate();
            OS_WaitIrq(TRUE, OS_IE_V_BLANK);
            Heap_Debug();
            dbs_DemoLoad();
        }
    }

    // Reads user data
    DTUD_Init();
    // Up to this point are startup operations


    // From this point are in-game Wi-Fi related operations
    while (1)
    {
        DrawConsole();

        ReadKeyData();

        update();

        dbs_DemoUpdate();
        OS_WaitIrq(TRUE, OS_IE_V_BLANK);
        Heap_Debug();
        dbs_DemoLoad();
    }

    OS_Terminate();
}
