#include <nitro.h>
#include <ninet/ip.h>
#include <dwc.h>

#include "main.h"
#include "init.h"
#include "utility.h"
#include "tp_input.h"
#include "wc.h"
#include "screen.h"
#include "dbs.h"
#include "mp.h"

#include "registerfriend.h"

extern DWCUserData stUserData; // Structure storing user data.
extern GameSequence gameSeqList[GAME_MODE_NUM];
extern GameControl stGameCnt;  // Game control data structure
extern KeyControl stKeyCnt;    // Key input control structure

// Friend key
static char stFriendKey[sizeof(DWCFriendData)];

// Friend data used for sample MP exchange
static DWCFriendData stSampleFriendData ATTRIBUTE_ALIGN(32);

//Data communications buffer
static u16* gSendBuf ATTRIBUTE_ALIGN(32);//Send buffer

/*---------------------------------------------------------------------------*
  Main function for registering friends
  *---------------------------------------------------------------------------*/
GameMode GameRegisterFriendMain(void)
{
    GameSequence *gameSeq = &gameSeqList[GAME_MODE_REG_FRIEND];
    GameMode returnSeq = GAME_MODE_NUM;
    RegFriendParam cntParam;
    int curIdx = 0;
    int friendIdx;
    int maxFigure;
    int validFlg;
    char tp_data;

    int i, j;

    tp_init();// Initialize touch pen input
    
    cntParam.step   = 0;
    cntParam.figure = 0;

    MI_CpuFill8( stFriendKey, '0', sizeof(stFriendKey));
    
    while (1){
        dbs_Print( 0, 0, "s:%d", DWC_GetMatchingState() );
        dbs_Print( 7, 0, "n:%d", DWC_GetNumConnectionHost() );
        dbs_Print( 0, 1, "w:%d", DWC_GetLinkLevel() );
        dbs_Print( 10,1, "p:%d", stGameCnt.userData.profileID );

        ReadKeyData();// Get key data

        // Escape if the next mode to go to has been set
        if (returnSeq != GAME_MODE_NUM){
            break;
        }
        
        ////////// Process the following key operations
        if (cntParam.step == 0){
            // Currently selecting command
            if (stKeyCnt.trg & PAD_BUTTON_A){
                // Select the menu with the A button
                returnSeq = gameSeq->menuList[curIdx].mode;
                stGameCnt.blocking = gameSeq->menuList[curIdx].blocking;

                if ( curIdx == 0 ){
                    // Initialize Wireless
                    DWC_CleanupInet();
                    
                    // Create data for MP communications
                    DWC_CreateExchangeToken(&stUserData, &stSampleFriendData);
                    gSendBuf = (u16*)(&stSampleFriendData);
                    
                    mp_match(gSendBuf, stGameCnt.friendList.keyList);//MP communications

                    friendIdx = GetAvailableFriendListIndex();// Omit identical MP data
                    for(i = 0; i < friendIdx; i++){
                        for(j = i+1; j < friendIdx; j++){
                            if(DWC_IsEqualFriendData( (const DWCAccFriendData*)&stGameCnt.friendList.keyList[i], (const DWCAccFriendData*)&stGameCnt.friendList.keyList[j])){
                                MI_CpuClear8(&stGameCnt.friendList.keyList[j], sizeof(DWCFriendData));
                            }
                        }
                    }
                    // Redisplay menu list
                    DispMenuMsgWithCursor(gameSeq, curIdx, &cntParam);
                }
                
                else if ((curIdx == 1) || (curIdx == 2)){
                    // Register friend using friend registration key input
                    // or delete friend data by specifying an index
                    cntParam.step   = 1;
                    cntParam.figure = 0;
                    cntParam.value  = 0;
                    
                    // Redisplay menu list
                    DispMenuMsgWithCursor(gameSeq, curIdx, &cntParam);
                    
                    if (curIdx == 1){
                        maxFigure = 12;  // Maximum 12 digits for friend registration key
                    }
                    else {
                        maxFigure = 2;   // The friend roster index is a maximum of 2 digits
                    }
                }
            }
            else if (stKeyCnt.trg & PAD_KEY_UP){
                // Move the cursor using up on the crosskey
                curIdx--;
                if (curIdx < 0) curIdx = gameSeq->numMenu-1;
                // Redisplay menu list
                DispMenuMsgWithCursor(gameSeq, curIdx, &cntParam);
            }
            else if (stKeyCnt.trg & PAD_KEY_DOWN){
                // Move the cursor using down on the crosskey
                curIdx++;
                if (curIdx >= gameSeq->numMenu) curIdx = 0;
                // Redisplay menu list
                DispMenuMsgWithCursor(gameSeq, curIdx, &cntParam);
            }
        }
        else {
            if(cntParam.figure < maxFigure){
                if((tp_data = tp_input())){
                    if(tp_data > 47){//If a numeric value from 0 to 9 is entered
                        stFriendKey[cntParam.figure] = tp_data;
                        if (cntParam.figure < maxFigure-1) cntParam.figure++;
                    }
                    else if(tp_data == 1){//If <- is entered
                        if (cntParam.figure > 0) cntParam.figure--;
                    }
                    else if(tp_data == 2){//-If >- is entered
                        if (cntParam.figure < maxFigure-1) cntParam.figure++;
                    }
                    // Data to be deleted
                    cntParam.value = (u32)((stFriendKey[0]-48)*10+(stFriendKey[1]-48));
                    DispMenuMsgWithCursor(gameSeq, curIdx, &cntParam);
                }
                // Display for touch pen input
                PrintString(10, 4, 0xf,  "-------------");
                PrintString(10, 5, 0xf,  "l   l   l   l");
                PrintString(10, 6, 0xf,  "l 7 l 8 l 9 l");
                PrintString(10, 7, 0xf,  "l   l   l   l");
                PrintString(10, 8, 0xf,  "l-----------l");
                PrintString(10, 9, 0xf,  "l   l   l   l");
                PrintString(10, 10, 0xf, "l 4 l 5 l 6 l");
                PrintString(10, 11, 0xf, "l   l   l   l");
                PrintString(10, 12, 0xf, "l-----------l");
                PrintString(10, 13, 0xf, "l   l   l   l");
                PrintString(10, 14, 0xf, "l 1 l 2 l 3 l");
                PrintString(10, 15, 0xf, "l   l   l   l");
                PrintString(10, 16, 0xf, "l-----------l");
                PrintString(10, 17, 0xf, "l   l   l   l");
                PrintString(10, 18, 0xf, "l 0 l<--l-->l");
                PrintString(10, 19, 0xf, "l   l   l   l");
                PrintString(10, 20, 0xf, "-------------");

                PrintString(2, 22, 0xf, "Set: PUSH A   Calcel: PUSH B");
            }
            
            // Currently entering a numeric value
            if (stKeyCnt.trg & PAD_BUTTON_A){
                // Add/delete friend using the A button
                if (curIdx == 1){
                    // Register friend
                    if (cntParam.value){
                        friendIdx = GetAvailableFriendListIndex();
                        if (friendIdx == -1){
                            OS_TPrintf("Failed to register new friend. Friend list is full.\n");
                        }
                        else {
                            if( DWC_CheckFriendKey( &stUserData, DWC_Acc_NumericKeyToFriendKey(stFriendKey)) ){// Validity check of friend registration key
                                DWC_CreateFriendKeyToken(&stGameCnt.friendList.keyList[friendIdx], DWC_Acc_NumericKeyToFriendKey(stFriendKey));// Register friend using friend registration key
                            }
                        }
                        validFlg = 1;
                    }
                    else {
                        validFlg = 0;  // Profile ID = 0 is invalid
                    }
                }
                else {
                    // Delete friend
                    if (cntParam.value < GAME_MAX_FRIEND_LIST){
                        // Delete friend from friend roster
                        DWC_DeleteBuddyFriendData(&stGameCnt.friendList.keyList[cntParam.value]);
                        validFlg = 1;
                    }
                    else {
                        validFlg = 0;  // Not on friend roster
                    }
                }

                if (validFlg){
                    // If input is valid, return to command selection
                    cntParam.step = 0;
                    // Redisplay menu list
                    DispMenuMsgWithCursor(gameSeq, curIdx, &cntParam);
                }
                ClearScreen();
            }
            else if (stKeyCnt.trg & PAD_BUTTON_B){
                MI_CpuFill8( stFriendKey, '0', sizeof(stFriendKey));
                // Return to command selection using the B button
                cntParam.step = 0;
                // Redisplay menu list
                DispMenuMsgWithCursor(gameSeq, curIdx, &cntParam);
                ClearScreen();
            }
            else if (stKeyCnt.trg & PAD_KEY_UP){
                // Increment numeric value during selection using the up/down buttons on the crosskey
                stFriendKey[cntParam.figure]++;
                if(stFriendKey[cntParam.figure] > 57){
                    stFriendKey[cntParam.figure] = '0';
                }
                //Data to be deleted
                cntParam.value = (u32)((stFriendKey[0]-48)*10+(stFriendKey[1]-48));
                // Redisplay menu list
                DispMenuMsgWithCursor(gameSeq, curIdx, &cntParam);
            }
            else if (stKeyCnt.trg & PAD_KEY_DOWN){
                // Increment numeric value during selection using the up/down buttons on the crosskey
                stFriendKey[cntParam.figure]--;
                if(stFriendKey[cntParam.figure] < 48){
                    stFriendKey[cntParam.figure] = '9';
                }
                //Data to be deleted
                cntParam.value = (u32)((stFriendKey[0]-48)*10+(stFriendKey[1]-48));
                // Redisplay menu list
                DispMenuMsgWithCursor(gameSeq, curIdx, &cntParam);
            }
            else if (stKeyCnt.trg & PAD_KEY_LEFT){
                // Move the number of selected digits using the left/right buttons on the crosskey
                if (cntParam.figure > 0 && cntParam.figure <= maxFigure-1 ) cntParam.figure--;
                // Redisplay menu list
                DispMenuMsgWithCursor(gameSeq, curIdx, &cntParam);
            }
            else if (stKeyCnt.trg & PAD_KEY_RIGHT){
                // Move the number of selected digits using the left/right buttons on the crosskey
                if (cntParam.figure >= 0 && cntParam.figure < maxFigure-1 ) cntParam.figure++;
                // Redisplay menu list
                DispMenuMsgWithCursor(gameSeq, curIdx, &cntParam);
            }
        }
        ////////// Key operation processing ends here

        dbs_DemoUpdate();
        OS_WaitIrq(TRUE, OS_IE_V_BLANK);
            Heap_Debug();
        dbs_DemoLoad();
        // Check for stack overflow
        OS_CheckStack(OS_GetCurrentThread());
    }

    return returnSeq;
}

/*---------------------------------------------------------------------------*
  Callback function for making additional displays during friend add/delete mode
 *---------------------------------------------------------------------------*/
void RegFriendModeDispCallback(int curIdx, void* param)
{
    RegFriendParam cntParam;
    int i;

    if (!param){
        // Called the first time with param = NULL
        cntParam.step = 0;
    }
    else {
        cntParam = *(RegFriendParam *)param;
    }

    OS_TPrintf("\n");

    if (cntParam.step == 0){
        // Display the friend roster during command selection
        DispFriendList(FALSE);
    }
    else {
        // Currently entering a numeric value
        if (curIdx == 1){
            // Register friend Currently entering a profile ID
            OS_TPrintf("Set friend key : ");
            for(i = 0; i < 12; i++){
                OS_TPrintf("%c", stFriendKey[i]);
            }
            OS_TPrintf("\n");

            // Display an underline to indicate the numeric value currently being selected
            for (i = 0; i < cntParam.figure; i++){
                OS_TPrintf(" ");
            }
            OS_TPrintf("                 -\n");
        }
        else {
            // Delete friend Currently entering a friend roster index
            DispFriendList(FALSE);  // Display the friend roster
            
            OS_TPrintf("Delete friend Index : ");
            for(i = 0; i < 2; i++){
                OS_TPrintf("%c", stFriendKey[i]);
            }
            OS_TPrintf("\n");

            for (i = 0; i < cntParam.figure; i++){
                OS_TPrintf(" ");
            }
            OS_TPrintf("                      -\n");
        }
    }
}

