/*===========================================================================*
 *                                                                           *
 *  ghttpCu@TvvO                                      *
 *                                                                           *
 *                                           Copyright 2005 Nintendo Co.,Ltd *
 *===========================================================================*/
#include <nitro.h>
#include <ninet/ip.h>
#include <dwc.h>

// Header file include
//-------------------------------------
#include "main.h"
#include "init.h"
#include "utility.h"

// Constant definitions
//---------------------------------------------------------
#define GAMENAME "dwctest"		//! Game name
#define USERID "12345"   		//! User ID
#define RECV_SIZE 256           //! Receive data size.

// Global variable declarations
//---------------------------------------------------------
DWCInetControl stConnCtrl;//! 
int		g_pending;		  //! Waiting for communication complete?
char*   buffer = NULL;           //! 

// Send/receive data format
//---------------------------------------------------------
typedef struct _tagGHTTPDATA{
	u8	tex[128];	//! Text
	u8	title[10];	//! Title
}GHTTPDATA;


// Function Prototype
//---------------------------------------------------------
static void POSTCallback(const char*, int, DWCGHTTPResult result, void*);
static void GETCallback(const char* buf, int buflen, DWCGHTTPResult result, void* param);
static void GETCallbackEx(const char* buf, int buflen, DWCGHTTPResult result, void* param);
static void GETProgressCallbackEx(DWCGHTTPState state, const char* buf, int len, int bytesReceived, int totalSize, void* param);
static BOOL StartIPMain(void);
static void VBlankIntr(void);

/*---------------------------------------------------------------------------*
  Function for allocating memory
 *---------------------------------------------------------------------------*/
static void*
AllocFunc( DWCAllocType name, u32   size, int align )
{
    void * ptr;
    OSIntrMode old;
    (void)name;
    (void)align;

    old = OS_DisableInterrupts();

    ptr = OS_AllocFromMain( size );

    OS_RestoreInterrupts( old );

    return ptr;
}

/*---------------------------------------------------------------------------*
  Function for freeing memory
 *---------------------------------------------------------------------------*/
static void
FreeFunc( DWCAllocType name, void* ptr,  u32 size  )
{
    OSIntrMode old;
    (void)name;
    (void)size;

    if ( !ptr ) return;

    old = OS_DisableInterrupts();

    OS_FreeToMain( ptr );

    OS_RestoreInterrupts( old );
}

//=============================================================================
/*!
 *	@brief	Callback when data send is complete
 *
 *	@param	buf		[in] Data buffer
 *	@param	buflen	[in] buf size
 *
 *	@retval	TRUE	Free buf
 *	@retval	FALSE   buf not freed
*/
//=============================================================================
static void POSTCallback(const char* buf, int buflen, DWCGHTTPResult result, void* param){
#ifdef SDK_FINALROM
#pragma unused(result)
#endif
#pragma unused(buf)
#pragma unused(buflen)
#pragma unused(param)

	g_pending = 0;

    OS_Printf("Post Result : %d\n", result);

}

//=============================================================================
/*!
 *	@brief	Callback when data receive is complete
 *
 *	@param	buf		[in] Data buffer
 *	@param	buflen	[in] buf size
 *  @param  param   [in] Parameter for callback
 *
 *	@retval	TRUE	Free buf
 *	@retval	FALSE   buf not freed
 */
//=============================================================================
static void GETCallback(const char* buf, int buflen, DWCGHTTPResult result, void* param){
#ifdef SDK_FINALROM
#pragma unused(result)
#endif
    char*	g_data;	//! Buffer for received data

    g_data = param;
    
	if(buf){
        MI_CpuCopy8(buf, g_data, (u32)buflen);
	}

	g_pending = 0;

    OS_Printf("Recv Result : %d\n", result);

}

//=============================================================================
/*!
 *	@brief	Callback used when data has been received (user receive buffer specification)
 *
 *	@param	buf		[in] Data buffer
 *	@param	buflen	[in] buf size
 *  @param  param   [in] Parameter for callback
 *
 *	@retval	TRUE	Buffer free
 *	@retval	FALSE   buffer not freed
 */
//=============================================================================
static void GETCallbackEx(const char* buf, int buflen, DWCGHTTPResult result, void* param){
#ifdef SDK_FINALROM
#pragma unused(result)
#pragma unused(buflen)
#endif
#pragma unused(param)

    if(buf){
        buffer = (char*)buf;
    }
    
    OS_Printf("Recv Result : %d\n", result);
    OS_Printf("buflen : %d\n", buflen);
    
	g_pending = 0;

}

//=============================================================================
/*!
 *	@brief	Callback when data receive is complete
 *
 *	@param	buf		[in] Data buffer
 *	@param	buflen	[in] buf size
 *  @param  param   [in] Parameter for callback
 *
 *	@retval	TRUE	Buffer free
 *	@retval	FALSE   buffer not freed
 */
//=============================================================================
static void GETProgressCallbackEx(DWCGHTTPState state, const char* buf, int len, int bytesReceived, int totalSize, void* param){
#ifdef SDK_FINALROM
#pragma unused(state)
#pragma unused(len)
#pragma unused(bytesReceived)
#pragma unused(totalSize)
#endif
#pragma unused(buf)
#pragma unused(param)

    OS_Printf("state: %d\n", state);
    OS_Printf("size : %d/%d\n", len, totalSize);
    OS_Printf("totalsize : %d/%d\n", bytesReceived, totalSize);
    
}

//=============================================================================
/*!
 *	@brief	Get IP
 *
 *	@retval	TRUE		Succeeded
 *	@retval	FALSE   	Failed
 */
//=============================================================================
static BOOL StartIPMain(void)
{
    DWC_InitInet( &stConnCtrl );

    DWC_ConnectInetAsync();
    // Wait for stable start.
    while ( !DWC_CheckInet() )
    {
        DWC_ProcessInet();

        OS_WaitVBlankIntr();

    }

    switch ( DWC_GetInetStatus() )
    {
    case DWC_CONNECTINET_STATE_ERROR:
        {
            // error display
            DWCError err;
            int errcode;
            err = DWC_GetLastError( &errcode );

            OS_TPrintf("   Error occurred %d %d.\n", err, errcode );
        }
        DWC_ClearError();

        DWC_CleanupInet();
        break;
    case DWC_CONNECTINET_STATE_FATAL_ERROR:
        {
            // error display
            DWCError err;
            int errcode;
            err = DWC_GetLastError( &errcode );

            OS_Panic("   Error occurred %d %d.\n", err, errcode );
        }
        break;

    case DWC_CONNECTINET_STATE_CONNECTED:
        OS_TPrintf("   CONNECTED!!!.\n");
        return TRUE;
    }
    return FALSE;
}

/*---------------------------------------------------------------------------*
  V-blank interrupt function
 *---------------------------------------------------------------------------*/
static void VBlankIntr(void)
{
    OS_SetIrqCheckFlag(OS_IE_V_BLANK);
}

//=============================================================================
/*!
 *	@brief	Main
 */
//=============================================================================
static u8 s_Work[ DWC_INIT_WORK_SIZE ] ATTRIBUTE_ALIGN( 32 );

void NitroMain()
{
	// For data
	GHTTPDATA data;
	int    len;
    char   g_data[1024];	//! Buffer for received data
    
    int req;//Request identifier
    
    DWCGHTTPPost post;

	// initialize
	//-------------------------------------------
	OS_Printf("initialize\n");

	OS_Init();
    OS_InitTick();
    OS_InitAlarm();
    RTC_Init();

    // Initial settings for checking for stack overflow
    OS_SetThreadStackWarningOffset(OS_GetCurrentThread(), 0x100);
    
    PXI_Init();
    
    Heap_Init();
	Alarm_Init();

    GX_Init();
    FS_Init( MI_DMA_MAX_NUM ); 

    GX_DispOff();
    GXS_DispOff();

    // Enable V-blank interrupt
    OS_SetIrqFunction(OS_IE_V_BLANK, VBlankIntr);
    (void)OS_EnableIrqMask(OS_IE_V_BLANK);
    (void)OS_EnableIrq();
    (void)OS_EnableInterrupts();
    (void)GX_VBlankIntr(TRUE);
	GX_SetDispSelect(GX_DISP_SELECT_SUB_MAIN);

    // Starts display
    OS_WaitVBlankIntr();              // Waiting the end of V-Blank interrupt
    GX_DispOn();
    GXS_DispOn();

    // Initialize DWC library
    DWC_Init( s_Work );
    
    // Specify debug display level
    DWC_SetReportLevel(DWC_REPORTFLAG_ALL);
    
    DWC_SetMemFunc( AllocFunc, FreeFunc );

	//Get IP
	while(!StartIPMain())
	{
		OS_Printf("error at StartIPMain()\n");	
	}
    
	// Initialize ghttp
	if(DWC_InitGHTTP(GAMENAME) != TRUE){
		OS_Printf("error at SCInitialize()\n");
	}

	// send data
	//-------------------------------------------
	memset(&data, 0, sizeof(data));
	len = sizeof(data);
    
	// *debug code* put some string
	strcpy((char*)data.tex, "this is a test message");
	strcpy((char*)data.title, "test");
    OS_Printf("[sent] %s\n", (char*)data.tex);
	OS_Printf("[sent] %s\n", (char*)data.title);

	// post data
	//-------------------------------------------
    DWC_GHTTPNewPost(&post);
    DWC_GHTTPPostAddString(&post, "data", (const char*)&data);
                           
	g_pending = 1;
	req = DWC_PostGHTTPData("URL", &post, POSTCallback, NULL);// URL specification required for posting
	do{
		DWC_ProcessGHTTP();
//	    OS_Printf("get request state: %d\n", DWC_GetGHTTPState(req));// Check communications status
	}while(g_pending);

	// receive data
	//-------------------------------------------
	g_pending = 1;
	req = DWC_GetGHTTPData("URL", GETCallback, (void*)g_data);// URL specification required for getting
	do{
		DWC_ProcessGHTTP();
//	    OS_Printf("get request state: %d\n", DWC_GetGHTTPState(req));// Check communications status
	}while(g_pending);

	// *debug code* put some string
	OS_Printf("[recv] %s\n", g_data);
	
	// receive data
	//-------------------------------------------
	g_pending = 1;
	req = DWC_GetGHTTPDataEx("URL", RECV_SIZE, FALSE, GETProgressCallbackEx, GETCallbackEx, NULL);// URL specification required for getting
	do{
		DWC_ProcessGHTTP();
//	    OS_Printf("get request state: %d\n", DWC_GetGHTTPState(req));// Check communications status
	}while(g_pending);

    if(buffer){
        // *debug code* put some string
        OS_Printf("[recv] %s\n", buffer);
        DWC_Free((DWCAllocType)0, (void*)buffer, 0);
    }
    // shutdown
	//-------------------------------------------
	OS_TPrintf("-- done --\n");

	DWC_ShutdownGHTTP();

	OS_Terminate();
}
